
public class Polynomial {
    private double[] coefficients;  
    private int deg;    
    
  
    public Polynomial(double a, int b) {
        coefficients = new double[b+1];
        coefficients[b] = a;
        deg = degree();
    }
	
	public Polynomial clone()
	{
		Polynomial p = new Polynomial(1,deg);
		
		
		for(int i=0; i <coefficients.length; i++)
		{
			p.coefficients[i] = this.coefficients[i];
		}
	
		return p;
	}

 
    public int degree() {
        int d = 0;
        for (int i = 0; i < coefficients.length; i++)
            if (coefficients[i] != 0) d = i;
        return d;
    }

    
    public Polynomial plus(Polynomial b) {
        Polynomial a = this;
        Polynomial c = new Polynomial(0, Math.max(a.deg, b.deg));
        for (int i = 0; i <= a.deg; i++) c.coefficients[i] += a.coefficients[i];
        for (int i = 0; i <= b.deg; i++) c.coefficients[i] += b.coefficients[i];
        c.deg = c.degree();
        return c;
    }

    
	public Polynomial times(double d)
	{
		for (int i = 0; i <= deg; i++) 
			coefficients[i] = coefficients[i]*d;
		
		return this;
	  
	}
    
	public Polynomial divide(Polynomial b)
	{
		
		Polynomial sum = new Polynomial(0,0);
		Polynomial temp = this.clone();
		
		while(temp.degree() !=0 && temp.degree() >= b.degree())
		{
			Polynomial p = b.clone();
			
			double divcoefficients = temp.coefficients[temp.degree()]/p.coefficients[p.degree()];
			Polynomial div = new Polynomial(divcoefficients,0);
			
			Polynomial c = new Polynomial(1, temp.degree() -p.degree());
			c = c.times(div);
			p = p.times(c);
			
			
			temp = temp.minus(p);
			sum = sum.plus(c);
				
		
		}
		
		return sum;
	}
	
    public Polynomial minus(Polynomial b)
    {
        Polynomial a = this;
        Polynomial c = new Polynomial(0, Math.max(a.deg, b.deg));
        for (int i = 0; i <= a.deg; i++) c.coefficients[i] += a.coefficients[i];
        for (int i = 0; i <= b.deg; i++) c.coefficients[i] -= b.coefficients[i];
        c.deg = c.degree();
        return c;
    }

    
    public Polynomial times(Polynomial b)
    {
        Polynomial a = this;
        Polynomial c = new Polynomial(0, a.deg + b.deg);
        for (int i = 0; i <= a.deg; i++)
            for (int j = 0; j <= b.deg; j++)
                c.coefficients[i+j] += (a.coefficients[i] * b.coefficients[j]);
        c.deg = c.degree();
        return c;
    }

   public double evaluate(double x) {
        double p = 0;
        for (int i = deg; i >= 0; i--)
            p = coefficients[i] + (x * p);
        return p;
    }

    
    public Polynomial differentiate() {
        if (deg == 0) return new Polynomial(0, 0);
        Polynomial deriv = new Polynomial(0, deg - 1);
        deriv.deg = deg - 1;
        for (int i = 0; i < deg; i++)
            deriv.coefficients[i] = (i + 1) * coefficients[i + 1];
        return deriv;
    }

   
    public String toString() {
        
		
		if (deg ==  0)
		{	
			return "" + coefficients[0];
        }
		if (deg ==  1) 
		{
			return coefficients[1] + "x + " + coefficients[0];
        }
		String s = coefficients[deg] + "x^" + deg;
        for (int i = deg-1; i >= 0; i--) {
            if      (coefficients[i] == 0) continue;
            else if (coefficients[i]  > 0) 
			{
			
				s = s + " + " + ( coefficients[i]);
            }
			else if (coefficients[i]  < 0) 
			{
				
				s = s + " - " + (-coefficients[i]);
            }
			if      (i == 1)
			{
		
				s = s + "x";
            }
			else if (i >  1) 
			{
			
				s = s + "x^" + i;
			}
		}
        return s;
	}
	
	
	
	public  double pow_int(double x, int n)
	{
		double result = 1;
		int i;

		for(i = 0; i < n; i++)
		{
			result *= x;
		}
		return result;
	}
	
	public double getRoot()
	{
	
		Polynomial n3 = this;
		//System.out.println(">>> Characteristic equation form for solving eigenvalue");
		//System.out.println(n3);
		//System.out.println("");
		
		double newVal;
		double old;
		double error;
		double oldVal =0;
		
		do
		{
		
			old = n3.evaluate(oldVal);
			Polynomial dif = n3.differentiate();
			newVal= oldVal - (old/dif.evaluate(oldVal));
			error = n3.evaluate(newVal);
			oldVal = newVal;
		}
		while (Math.abs(error) > 0.000001);

		return newVal;
		
	}
}