

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;


public class PolynomialMatrix
{
	Polynomial [][] mat;
	
	public Polynomial [][] convertToPolynomialMatrix(Matrix m)
	{
		Polynomial [][] p = new Polynomial[m.getRowNum()][m.getColNum()];
		
		for(int i=0; i < m.getRowNum(); i++)
		{
			for(int j=0; j< m.getColNum(); j++)
			{
				p[i][j] = new Polynomial(m.data[i][j],0);
			}
		}
		
		return p;
	}
	
	public Polynomial [][] convertToEigenCompatibleMatrix(Polynomial [][] p)
	{
		for(int i=0; i < p.length; i++)
		{
			for(int j=0; j<p[0].length; j++)
			{
				if(i==j)
				{
					Polynomial x = new Polynomial(-1,1);
					p[i][j] = x.plus(p[i][j]);
				}
			}
		}
		return p;
	}
	
	Polynomial determinant(Polynomial mat[][])
	{
		int n = mat.length;
		int i,j,rowCount,colCount, count=0;
		Polynomial [][] array = new Polynomial[n-1][n-1];
		Polynomial det = new Polynomial(0,0);
		if(n<1)
		{
			System.out.println("error");
		}
		if(n==1) return mat[0][0];
		if(n==2) 
		{	Polynomial a = mat[0][0].times(mat[1][1]);
			Polynomial b = mat[0][1].times(mat[1][0]);
			 return a.minus(b);
		}
		for(count=0; count<n; count++)
		{
		
			rowCount=0;
			for(i=1; i<n; i++)
			{
				colCount=0;
				for(j=0; j<n; j++)
				{
					if(j == count) continue;
					array[rowCount][colCount] = mat[i][j];
					colCount++;
				}
				rowCount++;
			}
			Polynomial pow = new Polynomial((double)Math.pow(-1, count),0);
			det = det.plus( pow.times(mat[0][count]).times(determinant(array)));
		
		}
		return det;
	}
	
	
	public static double format(double d)
	{
		long l = (int)Math.round(d * 1000); 
		d = l / 1000.0;
		return d;
	}
	
	
	public static void main(String[] args) { 
		
		double[][] d = { { 1, 2,3}, { 4,5,6 }, {7,8,9}};
		
		//System.out.println("!!! The below is steps for getting U (left vector)");
		//System.out.println("");
		System.out.println("Here is an original matrix!!!");
		Matrix o = new Matrix(d);
		o.show();
		System.out.println("\n>>> The below is SVD for the matrix above\n");
		
		Matrix l = left(d);
		System.out.println("****** Here is U (left vector)!!!!");
		l.show();
		System.out.println("");
		
		//System.out.println("!!! The below is steps for getting S (Singular values)");
		//System.out.println("");
		Matrix s = getS(d);
		System.out.println("****** Here is S (Singular values)!!!");
		s.show();
		System.out.println("");
		
		//Matrix r = right(d);
		System.out.println("****** Here is Transpose of V (right vector)!!!");
		Matrix A = new Matrix(d);
		Matrix AT = A.transpose();
		Matrix r = AT.times(l);
		r=r.getMultiNorm();
		
		r = getTransposeOfV(r);
		r.show();
		System.out.println("");
		System.out.println(">>> Getting the original vector A to verify by multiplying three matrices - U,S, Transpose of V");
		
		Matrix a = l.times(s).times(r);
		a.show();
		System.out.println("");
		System.out.println("Done!");
	}
	
	
	
	
	public static Matrix getV(List<Matrix> ev)
	{
		ListIterator<Matrix> mitr = ev.listIterator();
		
		Matrix V = new Matrix(ev.size(), ev.size());
		int j=0;
		while(mitr.hasNext())
		{
			Matrix e = mitr.next();
			for(int i=0;i<ev.size();i++)
			{
				V.data[i][j] = e.data[i][0];
			}
			
			j++;
			
		}
		
		return V;
	}
	
	
	public static Matrix getTransposeOfV(Matrix V)
	{
		return V.transpose();
	}
	 
	
	public static Matrix left(double [][]d)
	{
		List<Double> l = new LinkedList<Double>();  
		List<Matrix> lm = new LinkedList<Matrix>();
		List<Matrix> ev = new LinkedList<Matrix>();
		
	    Matrix D = new Matrix(d);
	    Matrix DT = D.transpose();
	       
	    D = D.times(DT);
	       
	    PolynomialMatrix pm = new PolynomialMatrix();
	    Polynomial [][] a = pm.convertToPolynomialMatrix(D);
	    Polynomial [][] b = pm.convertToEigenCompatibleMatrix(a);
	 
	    Polynomial c = pm.determinant(b);

		Polynomial copy = c.clone();
		int n = c.degree();
		
	    double x1 = copy.getRoot();
		
		l.add(format(x1));
	    n--;
	  
	    if(n>0)
		{
			for(int i=0; i <n; i++)
			{
				Polynomial c1 = copy.divide(new Polynomial(1,1).minus(new Polynomial(format(x1),0)));
		    	copy = c1;
		    	x1 = copy.getRoot();
		    	l.add(format(x1));
			}
		}    
	
	    ListIterator<Double> itr = l.listIterator();
		//System.out.println(">>> Getting eigenvalues corresponding to the characteristic equation form above.");
		/*
	    while(itr.hasNext())
		{
			 
			  System.out.println(itr.next());
		}
		*/
		System.out.println("");
		Collections.sort(l);
		Collections.reverse(l);
        //System.out.println(">>> Sorted eigenvalues in descending order. So the above order is reversed.");
		//System.out.println("");
		
		itr = l.listIterator();
		 
	    while(itr.hasNext())
	    {
	    	double val = itr.next();
	    	double[][] newMatrix = new double[a.length][a[0].length];
	    	
	    	for(int i=0; i < a.length; i++)
				{
					for(int j=0; j<a[0].length; j++)
					{
						 newMatrix[i][j] = b[i][j].evaluate(val);
					}
					
				}
	    	Matrix m = new Matrix(newMatrix);
	    	lm.add(m);
	    }
	    
	    
	    ListIterator<Matrix> mitr = lm.listIterator();
		int step=1;
		while(mitr.hasNext())
		{
			Matrix xn = mitr.next();
			//System.out.println("");
			//System.out.println(">>> Substituted egienvalue ( "+step +" out of 3) in the matrix.");
			//xn.show();
			//System.out.println("");
			Matrix fm = xn.solve();
			Matrix fs = fm.getNorm();
			//System.out.println(">>> The final result after normalizing.");
			//fs.show();
			
			ev.add(fs);
			step++;
		}
		Matrix g = getV(ev);
		return g;
	}
	
	
	public static Matrix right(double [][]d)
	{
		List<Double> l = new LinkedList<Double>();  
		List<Matrix> lm = new LinkedList<Matrix>();
		List<Matrix> ev = new LinkedList<Matrix>();
		
	    Matrix D = new Matrix(d);
	    Matrix DT = D.transpose();
	    D = DT.times(D);
	    PolynomialMatrix pm = new PolynomialMatrix();
	    Polynomial [][] a = pm.convertToPolynomialMatrix(D);
	        
	    Polynomial [][] b = pm.convertToEigenCompatibleMatrix(a);
	    Polynomial c = pm.determinant(b);
		
		Polynomial copy = c.clone();
		int n = c.degree();
		
		double x1 = copy.getRoot();
		
		l.add(format(x1));
	    n--;
	  
	    if(n>0)
		{
			for(int i=0; i <n; i++)
			{
				Polynomial c1 = copy.divide(new Polynomial(1,1).minus(new Polynomial(format(x1),0)));
		    	copy = c1;
		    	x1 = copy.getRoot();
		    	l.add(format(x1));
			}
			
		}    
	
	
		ListIterator<Double> itr = l.listIterator();
		 
		//System.out.println(">>> Getting eigenvalues corresponding to the characteristic equation form above.");
		while(itr.hasNext())
		{
			  System.out.println(itr.next());
		}
	
		Collections.sort(l);
		Collections.reverse(l);
		//System.out.println(">>> Sorted eigenvalues in descending order. So the above order is reversed.");
		//System.out.println("");
		
		
		itr = l.listIterator();
		 
	    while(itr.hasNext())
	    {
	    	double val = itr.next();
	    	double[][] newMatrix = new double[a.length][a[0].length];
	    	
	    	for(int i=0; i < a.length; i++)
				{
					for(int j=0; j<a[0].length; j++)
					{
						 newMatrix[i][j] = b[i][j].evaluate(val);
					}
					
				}
	    	Matrix m = new Matrix(newMatrix);
	    	lm.add(m);
	    }
	    
	   
	    
		ListIterator<Matrix> mitr = lm.listIterator();
		
		int step =1;
		while(mitr.hasNext())
		{
			Matrix xn = mitr.next();
		    //System.out.println("");
			//System.out.println(">>> Substituted egienvalue ( "+step +" out of 3) in the matrix.");
			Matrix fm = xn.solve();
		    Matrix fs = fm.getNorm();
			//System.out.println(">>> The final result after normalizing.");
			//fs.show();
			ev.add(fs);
			step++;
			
		}
		
		Matrix g = getV(ev);
		return g;		
	}
	
	public static Matrix getS(double [][]d)
	{
		
		List<Double> l = new LinkedList<Double>();  
		Matrix D = new Matrix(d);
	    Matrix DT = D.transpose();
	    D = D.times(DT);
	    
	    PolynomialMatrix pm = new PolynomialMatrix();
	    Polynomial [][] a = pm.convertToPolynomialMatrix(D);
	    Polynomial [][] b = pm.convertToEigenCompatibleMatrix(a);
	    Polynomial c = pm.determinant(b);
		Polynomial copy = c.clone();
		
		int n = c.degree();
		double x1 = copy.getRoot();
		
		l.add(format(x1));
	    n--;
	  
	    if(n>0)
		{
			for(int i=0; i <n; i++)
			{
				Polynomial c1 = copy.divide(new Polynomial(1,1).minus(new Polynomial(format(x1),0)));
		    	copy = c1;
		    	x1 = copy.getRoot();
		    	l.add(format(x1));
			}
			
		}    
	    
	    ListIterator<Double> itr = l.listIterator();
		Collections.sort(l);
		Collections.reverse(l);
	    
		double[][] S = new double[l.size()][l.size()];
		
		for(int i=0; i < l.size(); i++)
		{
			double val = itr.next();	
	    	for(int j=0; j<l.size(); j++)
			{
				if(i==j)
				{	
					//System.out.println(Math.sqrt(val));
					S[i][j]= Math.sqrt(val);
					
				}
			}
		}
	    	
	    Matrix s =new Matrix(S);
		return s;
	}
	
	
	public static Matrix left1(Matrix V, Matrix S, Matrix A)
	{
		
		List<Matrix> listU = new LinkedList<Matrix>();  
		
		int rows =0;
		for(int i=0; i<S.M; i++)
		{
		
			if(S.data[i][i]==0)
				continue;
			else
			{
				rows++;
				double a = 1/S.data[i][i];
				Matrix temp = A.Clone();
				int col = i;
				Matrix tempV = makeIndividual(V,col);
				Matrix c = temp.times(tempV);
				Matrix m =multiply(c,a);
				listU.add(m);
			}
				
		}
		
		Matrix l = new Matrix(rows, rows);
		ListIterator<Matrix> itr = listU.listIterator();
		for(int i=0; i < listU.size(); i++)
		{
			Matrix val = itr.next();	
			
			int r = val.M;
			for(int j=0; j<r; j++)
			{
				 l.data[j][i] = val.data[j][0];
			}
		}
		
		return l;
	}
	
	public static Matrix multiply(Matrix m, double n)
	{
		
		for( int i=0;i<m.M;i++){
             for(int j=0;j<m.N;j++){
                
            	 m.data[i][j] = n *m.data[i][j];
             }
             
		}
		return m;
	}
	
	public static Matrix makeIndividual(Matrix V, int i)
	{
		
		double [][] temp = new double[V.M][1];
		for(int r=0; r< V.M; r++)
		{
			temp[r][0]= V.data[r][i];
		}
	
		Matrix x = new Matrix(temp);
		return x;
	}	
}